@@ -3,7 +3,6 @@
 //! The major purpose of this crate is to let the user build their own YAML reader / builder / validator.
 //!
 //! This parser is not ensure about YAML spec but almost functions are well-implemented.
-//! (test case is [here](https://github.com/KmolYuan/yaml-peg-rs/blob/main/src/tests/test.yaml))
 //!
 //! The buffer reader has also not implemented, but the chunks can be read by sub-parsers.
 //!
@@ -11,11 +10,11 @@
 //!
 //! Function [`parse`] is used to parse YAML string into [`Node`] data structure,
 //! which has a data holder [`Yaml`].
-//! There also has multiple thread version corresponding to [`parse_arc`], [`ArcNode`], [`ArcYaml`].
+//! There also has multiple thread version corresponding to [`ArcNode`] and [`ArcYaml`].
 //! To get back as string, please use [`dump`] function.
 //!
 //! There are also have some macros for building [`NodeBase`] structure from Rust data.
-//! Especially [`node!`] / [`node_arc!`] macro, almost data can be built by the macro literally.
+//! Especially [`node!`] macro, almost data can be built by the macro literally.
 //!
 //! If you went to rise your own error message, [`indicated_msg`] might be a good choice.
 //!
@@ -42,14 +41,7 @@
 #![warn(missing_docs)]
 extern crate alloc;
 
-pub use crate::{
-    anchors::*,
-    dumper::dump,
-    indicator::*,
-    node::*,
-    parser::{parse, parse_arc},
-    yaml::*,
-};
+pub use crate::{anchors::*, dumper::dump, indicator::*, node::*, parser::parse, yaml::*};
 
 /// Create [`Node`] items literally.
 ///
@@ -66,10 +58,12 @@ pub use crate::{
 /// Arrays and maps can be created from this macro directly through brackets (`[]`, `{}`).
 ///
 /// ```
-/// use yaml_peg::{node, yaml_array, yaml_map};
+/// use yaml_peg::{node, Node};
 ///
-/// assert_eq!(node!([1, 2]), node!(yaml_array![1, 2]));
-/// assert_eq!(node!({1 => 2}), node!(yaml_map! { 1 => 2 }));
+/// let v = vec![Node::from(1), Node::from(2)];
+/// assert_eq!(node!([1, 2]), v.into_iter().collect());
+/// let m = vec![(Node::from(1), Node::from(2))];
+/// assert_eq!(node!({1 => 2}), m.into_iter().collect());
 /// ```
 ///
 /// The [`YamlBase::Anchor`] is also supported by the syntax:
@@ -77,99 +71,49 @@ pub use crate::{
 /// ```
 /// use yaml_peg::{node, YamlBase};
 ///
-/// assert_eq!(node!(YamlBase::Anchor("x".into())), node!(*"x"));
+/// assert_eq!(node!(YamlBase::Anchor("x".to_string())), node!(*"x"));
 /// ```
 ///
-/// For [`ArcNode`], please use [`node_arc!`], which has same API.
+/// This macro is use [`Node`] by default,
+/// to specify them, a "rc" or "arc" prefix token can choose the presentation.
+///
+/// ```
+/// use yaml_peg::{node, ArcNode};
+///
+/// assert_eq!(node!(arc ()), ArcNode::from(()));
+/// ```
 #[macro_export]
 macro_rules! node {
-    ([$($token:tt)*]) => {
-        $crate::node!($crate::yaml_array![$($token)*])
-    };
-    ({$($token:tt)*}) => {
-        $crate::node!($crate::yaml_map! { $($token)* })
-    };
-    (*$anchor:expr) => {
-        $crate::node!($crate::YamlBase::Anchor($anchor.into()))
-    };
-    ($yaml:expr) => {
-        $crate::Node::from($yaml)
-    };
-}
-
-/// Create [`ArcNode`] items literally.
-///
-/// The API is same as [`node!`] macro.
-///
-/// ```
-/// use yaml_peg::{node_arc as node, ArcNode};
-///
-/// assert_eq!(node!(()), ArcNode::from(()));
-/// ```
-#[macro_export]
-macro_rules! node_arc {
-    ([$($token:tt)*]) => {
-        $crate::node_arc!($crate::yaml_array![$($token)*])
-    };
-    ({$($token:tt)*}) => {
-        $crate::node_arc!($crate::yaml_map![$($token)*])
-    };
-    (*$anchor:expr) => {
-        $crate::node_arc!($crate::YamlBase::Anchor($anchor.into()))
-    };
-    ($yaml:expr) => {
-        $crate::ArcNode::from($yaml)
-    };
-}
-
-/// Create [`YamlBase::Array`] items literally.
-///
-/// All items will convert to [`NodeBase`] automatically.
-///
-/// ```
-/// use yaml_peg::{node, yaml_array, Yaml};
-///
-/// assert_eq!(
-///     yaml_array!["a", "b", "c"],
-///     Yaml::Array(vec![node!("a"), node!("b"), node!("c")])
-/// );
-/// ```
-#[macro_export]
-macro_rules! yaml_array {
-    ($v:expr; $n:expr) => {{
+    (@[$v:expr; $n:expr]) => {{
         extern crate alloc;
-        $crate::YamlBase::Array(alloc::vec![$crate::NodeBase::from($v); $n])
+        let v = alloc::vec![$crate::node!(@$v); $n];
+        $crate::node!(@$crate::YamlBase::Seq(v))
     }};
-    ($($v:expr),* $(,)?) => {{
+    (@[$($v:expr),* $(,)?]) => {{
         extern crate alloc;
-        $crate::YamlBase::Array(alloc::vec![$($crate::NodeBase::from($v)),*])
+        let v = alloc::vec![$($crate::node!(@$v)),*];
+        $crate::node!(@$crate::YamlBase::Seq(v))
     }};
-}
-
-/// Create [`YamlBase::Map`] items literally.
-///
-/// All items will convert to [`NodeBase`] automatically.
-///
-/// ```
-/// use yaml_peg::{node, yaml_map, Yaml};
-///
-/// assert_eq!(
-///     yaml_map! { "a" => "b", "c" => "d" },
-///     Yaml::Map(
-///         vec![(node!("a"), node!("b")), (node!("c"), node!("d"))]
-///             .into_iter()
-///             .collect()
-///     )
-/// );
-/// ```
-#[macro_export]
-macro_rules! yaml_map {
-    ($($k:expr => $v:expr),* $(,)?) => {{
+    (@{$($k:expr => $v:expr),* $(,)?}) => {{
         extern crate alloc;
-        $crate::YamlBase::Map(alloc::vec![
-            $(($crate::NodeBase::from($k), $crate::NodeBase::from($v))),*
-        ].into_iter().collect())
+        let m = alloc::vec![$(($crate::node!(@$k), $crate::node!(@$v))),*];
+        $crate::node!(@$crate::YamlBase::Map(m.into_iter().collect()))
     }};
+    (@*$anchor:expr) => {
+        $crate::node!(@$crate::YamlBase::Anchor($anchor.into()))
+    };
+    (@$yaml:expr) => {
+        $crate::NodeBase::from($yaml)
+    };
+    (arc $($tt:tt)+) => {
+        $crate::ArcNode::from($crate::node!(@$($tt)+))
+    };
+    (rc $($tt:tt)+) => {
+        $crate::Node::from($crate::node!(@$($tt)+))
+    };
+    ($($tt:tt)+) => {
+        $crate::node!(rc $($tt)+)
+    };
 }
 
 /// Create a custom anchor visitor.
@@ -189,7 +133,8 @@ macro_rules! yaml_map {
 macro_rules! anchors {
     ($($k:expr => $v:expr),* $(,)?) => {{
         extern crate alloc;
-        alloc::vec![$(($k.to_string(), $crate::NodeBase::from($v))),*].into_iter().collect::<$crate::AnchorBase<_>>()
+        let v = alloc::vec![$(($k.to_string(), $crate::node!(@$v))),*];
+        v.into_iter().collect::<$crate::AnchorBase<_>>()
     }};
 }
 
