@@ -3,6 +3,8 @@
 //! This parser is a simple greedy algorithm that returns result when
 //! matched successfully; try next or return error when mismatched.
 //!
+//! The [`Loader`] type can simply convert string into [`NodeBase`] type.
+//!
 //! Each pattern (the method of [`Parser`] type) is called "sub-parser",
 //! which returns a `Result<T, PError>` type, where `T` is the return type.
 //!
@@ -42,8 +44,8 @@
 //! + map terminator: The end of map is invalid, may caused by the last value
 //!   (like wrapped string).
 pub use self::{
-    base::{TakeOpt, DEFAULT_PREFIX},
-    error::PError,
+    base::{Parser, TakeOpt},
+    error::{PError, PResult},
 };
 use crate::{repr::Repr, *};
 use alloc::{
@@ -51,12 +53,16 @@ use alloc::{
     vec,
     vec::Vec,
 };
-use ritelinked::LinkedHashMap;
+use core::ops::{Deref, DerefMut};
 
 mod base;
-mod directive;
 mod error;
-mod grammar;
+
+macro_rules! tag_prefix {
+    () => {
+        "tag:yaml.org,2002:"
+    };
+}
 
 macro_rules! err {
     ($e:expr, $then:expr) => {
@@ -68,14 +74,22 @@ macro_rules! err {
     };
 }
 
-/// A PEG parser with YAML grammar, support UTF-8 characters.
+pub(crate) use tag_prefix;
+
+/// The default prefix of the YAML sub tag.
+pub const DEFAULT_PREFIX: &str = tag_prefix!();
+
+/// A parser with YAML grammar, support UTF-8 characters.
+///
+/// This loader will output YAML nodes with representation notation [`repr::Repr`].
+/// If you just want to use sub-parser, please see [`Parser`].
 ///
 /// A simple example for parsing YAML only:
 ///
 /// ```
-/// use yaml_peg::{node, parser::Parser};
+/// use yaml_peg::{node, parser::Loader};
 ///
-/// let n = Parser::new(b"true").parse().unwrap();
+/// let n = Loader::new(b"true").parse().unwrap();
 /// assert_eq!(n, vec![node!(true)]);
 /// ```
 ///
@@ -91,20 +105,23 @@ macro_rules! err {
 /// + Method [`Parser::forward`] is used to move on.
 /// + Method [`Parser::text`] is used to get the matched string.
 /// + Method [`Parser::backward`] is used to get back if mismatched.
-pub struct Parser<'a, R: Repr> {
-    doc: &'a [u8],
-    indent: Vec<usize>,
-    consumed: u64,
-    pub(crate) version_checked: bool,
-    pub(crate) tag: LinkedHashMap<String, String>,
-    /// Current position.
-    pub pos: usize,
-    /// Read position.
-    pub eaten: usize,
+pub struct Loader<'a, R: Repr> {
+    /// Parser base.
+    pub parser: Parser<'a>,
     /// A visitor of anchors.
     pub anchors: AnchorBase<R>,
 }
 
+impl<'a, R: Repr> Loader<'a, R> {
+    /// Create YAML loader includes a parser.
+    pub fn new(doc: &'a [u8]) -> Self {
+        Self {
+            parser: Parser::new(doc),
+            anchors: AnchorBase::new(),
+        }
+    }
+}
+
 /// The basic implementation.
 ///
 /// These sub-parser returns [`PError`], and failed immediately for [`PError::Terminate`].
@@ -120,11 +137,11 @@ pub struct Parser<'a, R: Repr> {
 /// # Parameter `inner`
 ///
 /// The `inner` parameter presents that the expression is in a **flow** expression.
-impl<R: Repr> Parser<'_, R> {
+impl<R: Repr> Loader<'_, R> {
     /// YAML entry point, return entire doc if exist.
-    pub fn parse(&mut self) -> Result<Seq<R>, PError> {
+    pub fn parse(&mut self) -> PResult<Seq<R>> {
         loop {
-            match self.context(Self::directive) {
+            match self.context(Parser::directive) {
                 Ok(()) => (),
                 Err(PError::Mismatch) => break,
                 Err(e) => return Err(e),
@@ -151,7 +168,7 @@ impl<R: Repr> Parser<'_, R> {
     }
 
     /// Match one doc block.
-    pub fn doc(&mut self) -> Result<NodeBase<R>, PError> {
+    pub fn doc(&mut self) -> PResult<NodeBase<R>> {
         self.ind_define(0)?;
         self.forward();
         let ret = self.scalar(0, false, false)?;
@@ -177,7 +194,7 @@ impl<R: Repr> Parser<'_, R> {
     }
 
     /// Match scalar.
-    pub fn scalar(&mut self, level: usize, nest: bool, inner: bool) -> Result<NodeBase<R>, PError> {
+    pub fn scalar(&mut self, level: usize, nest: bool, inner: bool) -> PResult<NodeBase<R>> {
         self.scalar_inner(|p| {
             if let Ok(s) = p.string_literal(level) {
                 Ok(YamlBase::Str(s))
@@ -193,13 +210,13 @@ impl<R: Repr> Parser<'_, R> {
     }
 
     /// Match flow scalar.
-    pub fn scalar_flow(&mut self, level: usize, inner: bool) -> Result<NodeBase<R>, PError> {
+    pub fn scalar_flow(&mut self, level: usize, inner: bool) -> PResult<NodeBase<R>> {
         self.scalar_inner(|p| p.scalar_term(level, inner))
     }
 
-    fn scalar_inner<F>(&mut self, f: F) -> Result<NodeBase<R>, PError>
+    fn scalar_inner<F>(&mut self, f: F) -> PResult<NodeBase<R>>
     where
-        F: Fn(&mut Self) -> Result<YamlBase<R>, PError>,
+        F: Fn(&mut Self) -> PResult<YamlBase<R>>,
     {
         let anchor = self.anchor().unwrap_or_default();
         if !anchor.is_empty() {
@@ -222,7 +239,7 @@ impl<R: Repr> Parser<'_, R> {
     }
 
     /// Match flow scalar terminal.
-    pub fn scalar_term(&mut self, level: usize, inner: bool) -> Result<YamlBase<R>, PError> {
+    pub fn scalar_term(&mut self, level: usize, inner: bool) -> PResult<YamlBase<R>> {
         let yaml = if let Ok(s) = self.float() {
             YamlBase::Float(s)
         } else if let Ok(s) = self.sci_float() {
@@ -234,7 +251,7 @@ impl<R: Repr> Parser<'_, R> {
         } else if let Ok(s) = self.string_quoted(b'\'', b"''") {
             YamlBase::Str(s)
         } else if let Ok(s) = self.string_quoted(b'"', b"\\\"") {
-            YamlBase::Str(Self::escape(&s))
+            YamlBase::Str(Parser::escape(&s))
         } else if let Ok(s) = self.string_plain(level, inner) {
             match s.as_str() {
                 "~" | "null" | "Null" | "NULL" => YamlBase::Null,
@@ -255,7 +272,7 @@ impl<R: Repr> Parser<'_, R> {
     }
 
     /// Match flow sequence.
-    pub fn seq_flow(&mut self, level: usize) -> Result<YamlBase<R>, PError> {
+    pub fn seq_flow(&mut self, level: usize) -> PResult<YamlBase<R>> {
         self.sym(b'[')?;
         let mut v = vec![];
         loop {
@@ -281,7 +298,7 @@ impl<R: Repr> Parser<'_, R> {
     }
 
     /// Match flow map.
-    pub fn map_flow(&mut self, level: usize) -> Result<YamlBase<R>, PError> {
+    pub fn map_flow(&mut self, level: usize) -> PResult<YamlBase<R>> {
         self.sym(b'{')?;
         let mut m = vec![];
         loop {
@@ -324,7 +341,7 @@ impl<R: Repr> Parser<'_, R> {
     }
 
     /// Match sequence.
-    pub fn seq(&mut self, level: usize, nest: bool) -> Result<YamlBase<R>, PError> {
+    pub fn seq(&mut self, level: usize, nest: bool) -> PResult<YamlBase<R>> {
         let mut v = vec![];
         loop {
             self.forward();
@@ -368,7 +385,7 @@ impl<R: Repr> Parser<'_, R> {
     }
 
     /// Match map.
-    pub fn map(&mut self, level: usize, nest: bool, inner: bool) -> Result<YamlBase<R>, PError> {
+    pub fn map(&mut self, level: usize, nest: bool, inner: bool) -> PResult<YamlBase<R>> {
         let mut m = vec![];
         loop {
             self.forward();
@@ -430,6 +447,20 @@ impl<R: Repr> Parser<'_, R> {
     }
 }
 
+impl<'a, R: Repr> Deref for Loader<'a, R> {
+    type Target = Parser<'a>;
+
+    fn deref(&self) -> &Self::Target {
+        &self.parser
+    }
+}
+
+impl<R: Repr> DerefMut for Loader<'_, R> {
+    fn deref_mut(&mut self) -> &mut Self::Target {
+        &mut self.parser
+    }
+}
+
 /// Parse YAML document into [`alloc::rc::Rc`] or [`alloc::sync::Arc`] data holder.
 /// Return an sequence of nodes and the anchors.
 ///
@@ -460,7 +491,7 @@ impl<R: Repr> Parser<'_, R> {
 /// })]);
 /// ```
 pub fn parse<R: Repr>(doc: &str) -> Result<(Seq<R>, AnchorBase<R>), String> {
-    let mut p = Parser::new(doc.as_bytes());
+    let mut p = Loader::new(doc.as_bytes());
     p.parse()
         .map_err(|e| e.into_error(doc))
         .map(|a| (a, p.anchors))
