@@ -1,5 +1,5 @@
 use super::SerdeError;
-use crate::{dump, repr::Repr, yaml_map, ArcNode, Array, Map, Node, NodeBase, YamlBase};
+use crate::{dump, node, repr::Repr, ArcNode, Map, Node, NodeBase, Seq, YamlBase};
 use alloc::string::{String, ToString};
 use core::marker::PhantomData;
 use serde::{
@@ -25,7 +25,7 @@ macro_rules! impl_end {
         $self.0.into()
     };
     (@map $self:ident) => {
-        yaml_map! { $self.1 => $self.0 }.into()
+        node!(@{$self.1 => $self.0})
     };
 }
 
@@ -115,7 +115,7 @@ pub fn to_node(any: impl Serialize) -> Result<Node, SerdeError> {
 ///
 /// ```
 /// use serde::Serialize;
-/// use yaml_peg::{serialize::to_arc_node, node_arc};
+/// use yaml_peg::{serialize::to_arc_node, node};
 ///
 /// #[derive(Serialize)]
 /// struct Member<'a> {
@@ -125,7 +125,7 @@ pub fn to_node(any: impl Serialize) -> Result<Node, SerdeError> {
 /// }
 ///
 /// let officer = Member { name: "Bob", married: true, age: 46 };
-/// let officer_yaml = node_arc!({
+/// let officer_yaml = node!(arc{
 ///     "name" => "Bob",
 ///     "married" => true,
 ///     "age" => 46,
@@ -250,15 +250,15 @@ impl<R: Repr> Serializer for NodeSerializer<R> {
     where
         T: Serialize + ?Sized,
     {
-        Ok(yaml_map! { variant => value.serialize(NodeSerializer(PhantomData))? }.into())
+        Ok(node!(@{variant => value.serialize(NodeSerializer(PhantomData))?}))
     }
 
     fn serialize_seq(self, len: Option<usize>) -> Result<Self::SerializeSeq, Self::Error> {
-        let array = match len {
-            Some(n) => Array::with_capacity(n),
-            None => Array::new(),
+        let seq = match len {
+            Some(n) => Seq::with_capacity(n),
+            None => Seq::new(),
         };
-        Ok(SeqSerializer(array))
+        Ok(SeqSerializer(seq))
     }
 
     fn serialize_tuple(self, len: usize) -> Result<Self::SerializeTuple, Self::Error> {
@@ -280,7 +280,7 @@ impl<R: Repr> Serializer for NodeSerializer<R> {
         variant: &'static str,
         len: usize,
     ) -> Result<Self::SerializeTupleVariant, Self::Error> {
-        Ok(TupleVariant(Array::with_capacity(len), variant))
+        Ok(TupleVariant(Seq::with_capacity(len), variant))
     }
 
     fn serialize_map(self, len: Option<usize>) -> Result<Self::SerializeMap, Self::Error> {
@@ -320,8 +320,8 @@ impl<R: Repr> Serializer for NodeSerializer<R> {
     }
 }
 
-struct SeqSerializer<R: Repr>(Array<R>);
-struct TupleVariant<R: Repr>(Array<R>, &'static str);
+struct SeqSerializer<R: Repr>(Seq<R>);
+struct TupleVariant<R: Repr>(Seq<R>, &'static str);
 struct MapSerializer<R: Repr>(Map<R>, Option<NodeBase<R>>);
 struct StructSerializer<R: Repr>(Map<R>);
 struct StructVariant<R: Repr>(Map<R>, &'static str);
